@@ -29,7 +29,6 @@ def editor_opened(editor):
     handlers[editor] = h
 
 def editor_closed(editor):
-    handlers[editor].close()
     del handlers[editor]
 
 def quit():
@@ -38,6 +37,10 @@ def quit():
         outline_dialog.window.destroy()
         del outline_dialog
 
+    import plugin
+    for v in plugin.project_managers.values():
+        v.close()
+
 def goto_definition(editor):
     try:
         h = handlers[editor]
