@@ -18,7 +18,7 @@ def idle(callable, *args, **kwargs):
 
 def save_file(filename, data, encoding):
     tmpfilename = realpath(filename) + '.bak'
-    
+
     try:
         f = open(tmpfilename, 'w')
     except IOError:
@@ -26,6 +26,8 @@ def save_file(filename, data, encoding):
         if not exists(dname):
             os.makedirs(dname, mode=0755)
             f = open(tmpfilename, 'w')
+        else:
+            raise
                     
     f.write(data.encode(encoding))
     f.close()
@@ -101,6 +103,14 @@ def lazy_property(func):
         
     return property(inner)
 
+def set_activate_the_one_item(entry, treeview):
+    def activate(*args):
+        if len(treeview.get_model()) == 1:
+            treeview.set_cursor((0,))
+            treeview.row_activated((0,), treeview.get_column(0))
+        
+    entry.connect('activate', activate)
+
 
 class BuilderAware(object):
     def __init__(self, glade_file):
